@@ -7,7 +7,7 @@ CGI::Ex - CGI utility suite - makes powerful application writing fun and easy
 =cut
 
 ###----------------------------------------------------------------###
-#  Copyright 2003-2012 - Paul Seamons                                #
+#  Copyright 2003-2014 - Paul Seamons                                #
 #  Distributed under the Perl Artistic License without warranty      #
 ###----------------------------------------------------------------###
 
@@ -24,7 +24,7 @@ use vars qw($VERSION
 use base qw(Exporter);
 
 BEGIN {
-    $VERSION               = '2.38';
+    $VERSION               = '2.41';
     $PREFERRED_CGI_MODULE  ||= 'CGI';
     @EXPORT = ();
     @EXPORT_OK = qw(get_form
@@ -294,6 +294,7 @@ sub content_typed {
 sub location_bounce {
     my ($self, $loc) = ($#_ == 1) ? (@_) : (undef, shift);
     $self = __PACKAGE__->new if ! $self;
+    $loc =~ s{(\s)}{sprintf("%%%02X", ord $1)}xge if $loc;
 
     if ($self->content_typed) {
         if ($DEBUG_LOCATION_BOUNCE) {
@@ -1057,6 +1058,6 @@ This module may be distributed under the same terms as Perl itself.
 
 =head1 AUTHOR
 
-Paul Seamons <perl at seamons dot com>
+Paul Seamons <paul@seamons.com>
 
 =cut
