@@ -7,15 +7,14 @@ CGI::Ex::Conf - Conf Reader/Writer for many different data format types
 =cut
 
 ###----------------------------------------------------------------###
-#  Copyright 2003-2012 - Paul Seamons                                #
+#  Copyright 2003-2014 - Paul Seamons                                #
 #  Distributed under the Perl Artistic License without warranty      #
 ###----------------------------------------------------------------###
 
 use strict;
 use base qw(Exporter);
 use Carp qw(croak);
-use vars qw($VERSION
-            @DEFAULT_PATHS
+use vars qw(@DEFAULT_PATHS
             $DEFAULT_EXT
             %EXT_READERS
             %EXT_WRITERS
@@ -29,8 +28,6 @@ use vars qw($VERSION
             );
 @EXPORT_OK = qw(conf_read conf_write in_cache);
 
-$VERSION = '2.38';
-
 $DEFAULT_EXT = 'conf';
 
 %EXT_READERS = (''         => \&read_handler_yaml,
@@ -925,7 +922,7 @@ This module may be distributed under the same terms as Perl itself.
 
 =head1 AUTHOR
 
-Paul Seamons <perl at seamons dot com>
+Paul Seamons <paul@seamons.com>
 
 =cut
 
