@@ -2,22 +2,21 @@ package CGI::Ex::Validate;
 
 ###---------------------###
 #  See the perldoc in CGI/Ex/Validate.pod
-#  Copyright 2003-2012 - Paul Seamons
+#  Copyright 2003-2014 - Paul Seamons
 #  Distributed under the Perl Artistic License without warranty
 
 use strict;
 use Carp qw(croak);
 
-our $VERSION  = '2.38';
+our $VERSION  = '2.41';
 our $QR_EXTRA = qr/^(\w+_error|as_(array|string|hash)_\w+|no_\w+)/;
 our @UNSUPPORTED_BROWSERS = (qr/MSIE\s+5.0\d/i);
 our $JS_URI_PATH;
 our $JS_URI_PATH_VALIDATE;
 
 sub new {
-    my $class = shift || croak "Usage: ".__PACKAGE__."->new";
-    my $self  = ref($_[0]) ? shift : {@_};
-    return bless $self, $class;
+    my $class = shift;
+    return bless ref($_[0]) ? shift : {@_}, $class;
 }
 
 sub cgix { shift->{'cgix'} ||= do { require CGI::Ex; CGI::Ex->new } }
@@ -53,32 +52,50 @@ sub validate {
             next;
         }
         $found = 1;
-        my $field = $ref->{'field'} || die "Missing field key during normal validation";
-        if (! $checked{$field}++) {
-            $self->{'was_checked'}->{$field} = 1;
-            $self->{'was_valid'}->{$field} = 1;
-            $self->{'had_error'}->{$field} = 0;
-        }
-        local $ref->{'was_validated'} = 1;
-        my $err = $self->validate_buddy($form, $field, $ref);
-        if ($ref->{'was_validated'} && $what_was_validated) {
-            push @$what_was_validated, $ref;
-        } else {
-            $self->{'was_valid'}->{$field} = 0;
-        }
+        my $key = $ref->{'field'} || die "Missing field key during normal validation";
 
-        # test the error - if errors occur allow for OR - if OR fails use errors from first fail
-        if ($err) {
-            $self->{'was_valid'}->{$field} = 0;
-            $self->{'had_error'}->{$field} = 0;
-            if ($i < $#$fields && ! ref($fields->[$i + 1]) && $fields->[$i + 1] eq 'OR') {
-                $hold_error = $err;
-            } else {
-                push @errors, $hold_error ? @$hold_error : @$err;
-                $hold_error = undef;
+        # allow for field names that contain regular expressions
+        my @keys;
+        if ($key =~ m/^(!\s*|)m([^\s\w])(.*)\2([eigsmx]*)$/s) {
+            my ($not,$pat,$opt) = ($1,$3,$4);
+            $opt =~ tr/g//d;
+            die "The e option cannot be used on validation keys on field $key" if $opt =~ /e/;
+            foreach my $_key (sort keys %$form) {
+                next if ($not && $_key =~ m/(?$opt:$pat)/) || (! $not && $_key !~ m/(?$opt:$pat)/);
+                push @keys, [$_key, [undef, $1, $2, $3, $4, $5]];
             }
         } else {
-            $hold_error = undef;
+            @keys = ([$key]);
+        }
+
+        foreach my $r (@keys) {
+            my ($field, $ifs_match) = @$r;
+            if (! $checked{$field}++) {
+                $self->{'was_checked'}->{$field} = 1;
+                $self->{'was_valid'}->{$field} = 1;
+                $self->{'had_error'}->{$field} = 0;
+            }
+            local $ref->{'was_validated'} = 1;
+            my $err = $self->validate_buddy($form, $field, $ref, $ifs_match);
+            if ($ref->{'was_validated'}) {
+                push @$what_was_validated, $ref if $what_was_validated;
+            } else {
+                $self->{'was_valid'}->{$field} = 0;
+            }
+
+            # test the error - if errors occur allow for OR - if OR fails use errors from first fail
+            if ($err) {
+                $self->{'was_valid'}->{$field} = 0;
+                $self->{'had_error'}->{$field} = 0;
+                if ($i < $#$fields && ! ref($fields->[$i + 1]) && $fields->[$i + 1] eq 'OR') {
+                    $hold_error = $err;
+                } else {
+                    push @errors, $hold_error ? @$hold_error : @$err;
+                    $hold_error = undef;
+                }
+            } else {
+                $hold_error = undef;
+            }
         }
     }
     push(@errors, @$hold_error) if $hold_error; # allow for final OR to work
@@ -179,7 +196,7 @@ sub check_conditional {
         my $ref = $ifs->[$i];
         if (! ref $ref) {
             if ($ref eq 'OR') {
-                $i ++ if $found; # if found skip the OR altogether
+                $i++ if $found; # if found skip the OR altogether
                 $found = 1; # reset
                 next;
             } else {
@@ -201,6 +218,7 @@ sub check_conditional {
         $field =~ s/\$(\d+)/defined($ifs_match->[$1]) ? $ifs_match->[$1] : ''/eg if $ifs_match;
 
         my $errs = $self->validate_buddy($form, $field, $ref);
+
         $found = 0 if $errs;
     }
     return $found;
@@ -226,8 +244,7 @@ sub validate_buddy {
         die "The e option cannot be used on validation keys on field $field" if $opt =~ /e/;
         foreach my $_field (sort keys %$form) {
             next if ($not && $_field =~ m/(?$opt:$pat)/) || (! $not && $_field !~ m/(?$opt:$pat)/);
-            my @match = (undef, $1, $2, $3, $4, $5); # limit to the matches
-            my $errs = $self->validate_buddy($form, $_field, $field_val, \@match);
+            my $errs = $self->validate_buddy($form, $_field, $field_val, [undef, $1, $2, $3, $4, $5]);
             push @errors, @$errs if $errs;
         }
         return @errors ? \@errors : 0;
@@ -237,6 +254,7 @@ sub validate_buddy {
     if ($field_val->{'had_error'}   && ! $self->{'had_error'}->{$field})   { return [[$field, 'had_error',   $field_val, $ifs_match]]; }
     if ($field_val->{'was_checked'} && ! $self->{'was_checked'}->{$field}) { return [[$field, 'was_checked', $field_val, $ifs_match]]; }
 
+
     # allow for default value
     if (defined($field_val->{'default'})
         && (!defined($form->{$field})
@@ -406,10 +424,13 @@ sub validate_buddy {
             if ($field2 =~ m/^([\"\'])(.*)\1$/) {
                 my $test = $2;
                 $success = (defined($value) && $value eq $test);
-            } elsif (exists($form->{$field2}) && defined($form->{$field2})) {
-                $success = (defined($value) && $value eq $form->{$field2});
-            } elsif (! defined($value)) {
-                $success = 1; # occurs if they are both undefined
+            } else {
+                $field2 =~ s/\$(\d+)/defined($ifs_match->[$1]) ? $ifs_match->[$1] : ''/eg if $ifs_match;
+                if (exists($form->{$field2}) && defined($form->{$field2})) {
+                    $success = (defined($value) && $value eq $form->{$field2});
+                } elsif (! defined($value)) {
+                    $success = 1; # occurs if they are both undefined
+                }
             }
             if ($not ? $success : ! $success) {
                 return [] if $self->{'_check_conditional'};
