@@ -1,4 +1,4 @@
-// Copyright 2003-2012 - Paul Seamons - ver 2.37
+// Copyright 2003-2014 - Paul Seamons - ver 2.37
 // Distributed under the Perl Artistic License without warranty
 // See perldoc CGI::Ex::Validate for usage
 
@@ -123,9 +123,14 @@ function v_clean_field_val (field_val, N_level) {
   } else if (k.match(/^custom_js_?\d*$/)) {
    if (typeof(v) == 'string' && v.match(/^\s*function\s*\(/)) eval("field_val[k] = "+v);
   } else if (k.match(/^(validate|required)_if_?\d*$/)) {
-    if (typeof(v) == 'string' || ! v.length) v = field_val[k] = [v];
-    var deps = v_clean_cond(v, N_level);
-    for (var k in deps) field_val.deps[k] = 2;
+   if (typeof(v) == 'string' || ! v.length) v = field_val[k] = [v];
+   var deps = v_clean_cond(v, N_level);
+   for (var k in deps) field_val.deps[k] = 2;
+  } else if (k.match(/^equals_?\d*$/)) {
+   if (!/^[\"\']/.test(field_val[k])) {
+    var deps = v_clean_cond([field_val[k].replace(/^!\s*/,'')], N_level);
+    for (var k in deps) field_val.deps[k] = 3;
+   }
   }
  }
 }
@@ -168,7 +173,7 @@ function v_validate (form, val_hash) {
 
  var is_found  = 1;
  var errors = [];
- var hold_error;
+ var hold_err;
 
  var chk = {};
  for (var j = 0; j < fields.length; j++) {
@@ -179,26 +184,31 @@ function v_validate (form, val_hash) {
    continue;
   }
   is_found = 1;
-  var f = ref.field;
-  if (! chk[f]) {
-   chk[f] = 1;
-   val_hash['group was_checked'][f] = 1;
-   val_hash['group was_valid'][f]   = 1;
-   val_hash['group had_error'][f]   = 0;
-  }
-  var err = v_validate_buddy(form, f, ref, val_hash);
-  if (err.length) {
-   val_hash['group had_error'][f] = 1;
-   val_hash['group was_valid'][f] = 0;
-   if (j <= fields.length && typeof(fields[j + 1] != 'object') && fields[j + 1] == 'OR') {
-    hold_error = err;
+  var names = v_field_names(form, ref.field);
+  if (!names) names = [[ref.field, null]];
+  for (var i = 0; i < names.length; i++) {
+   var f = names[i][0];
+   var ifs_match = names[i][1];
+   if (! chk[f]) {
+    chk[f] = 1;
+    val_hash['group was_checked'][f] = 1;
+    val_hash['group was_valid'][f]   = 1;
+    val_hash['group had_error'][f]   = 0;
+   }
+   var err = v_validate_buddy(form, f, ref, val_hash, ifs_match);
+   if (err.length) {
+    val_hash['group had_error'][f] = 1;
+    val_hash['group was_valid'][f] = 0;
+    if (j <= fields.length && typeof(fields[j + 1] != 'object') && fields[j + 1] == 'OR') {
+     hold_err = err;
+    } else {
+     if (hold_err) err = hold_err;
+     for (var k = 0; k < err.length; k++) errors.push(err[k]);
+     hold_err = '';
+    }
    } else {
-    if (hold_error) err = hold_error;
-    for (var k = 0; k < err.length; k++) errors.push(err[k]);
-    hold_error = '';
+    hold_err = '';
    }
-  } else {
-   hold_error = '';
   }
  }
 
@@ -264,27 +274,35 @@ function v_field_order (field_val) {
  return o.sort();
 }
 
+function v_field_names (form, field) {
+ var m = field.match(/^(!\s*|)m([^\s\w])(.*)\2([eigsmx]*)$/);
+ if (!m) return;
+ var fields = [];
+ var not = m[1];
+ var pat = m[3];
+ var opt = m[4];
+ if (opt.indexOf('e') != -1) { v_error("The e option cannot be used on field "+field); return [] }
+ opt = opt.replace(/[sg]/g,'');
+ var reg = new RegExp(pat, opt);
+
+ for (var i = 0; i < form.elements.length; i++) {
+  var _field = form.elements[i].name;
+  if (_field && (not && ! (m = _field.match(reg))) || (m = _field.match(reg))) fields.push([_field, m]);
+ }
+ return fields;
+}
+
 function v_validate_buddy (form, field, field_val, val_hash, ifs_match) {
  var errors = [];
  if (! form.elements || field_val.exclude_js) return [];
  var types = field_val.order || v_field_order(field_val);
-
  var m;
- if (m = field.match(/^(!\s*|)m([^\s\w])(.*)\2([eigsmx]*)$/)) {
-  var not = m[1];
-  var pat = m[3];
-  var opt = m[4];
-  if (opt.indexOf('e') != -1) { v_error("The e option cannot be used on field "+field); return [] }
-  opt = opt.replace(/[sg]/g,'');
-  var reg = new RegExp(pat, opt);
 
-  for (var i = 0; i < form.elements.length; i++) {
-   var _field = form.elements[i].name;
-   if (! _field) continue;
-   if ( (not && ! (m = _field.match(reg))) || (m = _field.match(reg))) {
-    var err = v_validate_buddy(form, _field, field_val, val_hash, m);
-    for (var j = 0; j < err.length; j++) errors.push(err[j]);
-   }
+ var names = v_field_names(form, field);
+ if (names) {
+  for (var i = 0; i < names.length; i++) {
+   var err = v_validate_buddy(form, names[i][0], field_val, val_hash, names[i][1]);
+   for (var j = 0; j < err.length; j++) errors.push(err[j]);
   }
   return errors;
  }
@@ -300,7 +318,7 @@ function v_validate_buddy (form, field, field_val, val_hash, ifs_match) {
      && (typeof(_value) == 'undefined'
          || (typeof(_value) == 'object' && _value.length == 0)
          || ! _value.length)) {
-  _value = field_val['default'];
+  _value = ''+field_val['default'];
   modified = 1;
  }
 
@@ -437,7 +455,10 @@ function v_validate_buddy (form, field, field_val, val_hash, ifs_match) {
     if (m = _fv.match(/^([\"\'])(.*)\1$/)) {
      if (value == m[2]) success = 1;
     } else {
-     var value2 = v_get_form_value(form[_fv]);
+     var _fv2 = _fv.replace(/\$(\d+)/g, function (all, N) {
+      return (typeof(ifs_match) != 'object' || typeof(ifs_match[N]) == 'undefined') ? '' : ifs_match[N];
+     });
+     var value2 = v_get_form_value(form[_fv2]);
      if (typeof(value2) == 'undefined') value2 = '';
      if (value == value2) success = 1;
     }
@@ -597,7 +618,6 @@ function v_set_form_value (el, values, form) {
   return;
  }
  if (! type) return;
- if (type.match(/(hidden|password|text|textarea|submit)/)) return el.value = values[0];
  if (type.indexOf('select') != -1) {
    if (el.length) for (var i = 0; i < el.length; i++) el[i].selected = (el[i].value == values[0]) ? true : false;
    return;
@@ -607,9 +627,7 @@ function v_set_form_value (el, values, form) {
   return el.checked = f ? true : false;
  }
  if (type == 'file') return;
-
- alert('Unknown form type for '+el.name+': '+type);
- return;
+ return el.value = values[0];
 }
 
 function v_set_disable (el, disable) {
@@ -638,17 +656,13 @@ function v_get_form_value (el, form) {
   return a;
  }
  if (! type) return '';
- if (type.match(/(hidden|password|text|textarea|submit)/)) return el.value;
  if (type.indexOf('select') != -1) {
   if (! el.length) return '';
   if (el.selectedIndex == -1) return '';
   return el[el.selectedIndex].value;
  }
  if (type == 'checkbox' || type == 'radio') return el.checked ? el.value : '';
- if (type == 'file') return el.value;
-
- alert('Unknown form type for '+el.name+': '+type);
- return '';
+ return el.value;
 }
 
 function v_find_val () {
@@ -965,8 +979,20 @@ document.check_form = function (form, val_hash) {
   var h = {};
   _add = function (k, v) { if (! h[k]) h[k] = []; h[k].push(v) };
   for (var i = 0; i < clean.fields.length; i++) {
-   _add(clean.fields[i].field, clean.fields[i]);
-   for (var j in clean.fields[i].deps) if (j != clean.fields[i].field) _add(j, clean.fields[i]);
+   var k = clean.fields[i].field;
+   var names = v_field_names(form,k);
+   if (!names) names = [[k,null]];
+    for (var ii = 0; ii < names.length; ii++) {
+     var k = names[ii][0];
+     var ifs_match = names[ii][1];
+     _add(k, [clean.fields[i],k,ifs_match]);
+     for (var j in clean.fields[i].deps) {
+      if (ifs_match) j = j.replace(/\$(\d+)/g, function (all, N) {
+       return (typeof(ifs_match) != 'object' || typeof(ifs_match[N]) == 'undefined') ? '' : ifs_match[N];
+      });
+      if (j != k) _add(j, [clean.fields[i],k,ifs_match]);
+     }
+    }
   }
   for (var k in h) {
    if (!h.hasOwnProperty(k)) continue;
@@ -974,7 +1000,7 @@ document.check_form = function (form, val_hash) {
    if (! el) return v_error("No form element by the name "+k);
    var _change = !types.change ? 0 : typeof(types.change) == 'object' ? types.change[k] : 1;
    var _blur   = !types.blur   ? 0 : typeof(types.blur)   == 'object' ? types.blur[k]   : 1;
-   v_el_attach(el, h[k], form, val_hash, _change, _blur);
+   v_el_attach(el, h[k], form, val_hash, _change, _blur, ifs_match);
   }
  }
 
@@ -986,7 +1012,7 @@ document.check_form = function (form, val_hash) {
  if (types.load) { v_event = 'load'; document.validate(form) }
 }
 
-function v_el_attach (el, fvs, form, val_hash, _change, _blur) {
+function v_el_attach (el, fvs, form, val_hash, _change, _blur, ifs_match) {
  if (!_change && !_blur) return;
  if (! el.type) {
   if (el.length) for (var i = 0; i < el.length; i++) v_el_attach(el[i], fvs, form, val_hash, _change, _blur);
@@ -999,21 +1025,28 @@ function v_el_attach (el, fvs, form, val_hash, _change, _blur) {
   var f = {};
   var chk = {};
   for (var i = 0; i < fvs.length; i++) {
-   var field_val = fvs[i];
-   var k = field_val.field;
+   var field_val = fvs[i][0];
+   var k = fvs[i][1];
+   var ifs_match = fvs[i][2];
    if (! chk[k]) {
     chk[k] = 1;
     val_hash['group was_checked'][k] = 1;
     val_hash['group was_valid'][k]   = 1;
     val_hash['group had_error'][k]   = 0;
    }
-   var _e = v_validate_buddy(form, k, field_val, val_hash);
+   var _e = v_validate_buddy(form, k, field_val, val_hash, ifs_match);
    if (_e.length) {
     val_hash['group had_error'][k] = 1;
     val_hash['group was_valid'][k] = 0;
     for (var j = 0; j < _e.length; j++) e.push(_e[j]);
    }
-   f[field_val.delegate_error || field_val.field] = _e.length ? 0 : 1;
+   if (field_val.delegate_error) {
+    k = field_val.delegate_error;
+    if (ifs_match) k = k.replace(/\$(\d+)/g, function (all, N) {
+     return (typeof(ifs_match) != 'object' || typeof(ifs_match[N]) == 'undefined') ? '' : ifs_match[N];
+    });
+   }
+   f[k] = _e.length ? 0 : 1;
   }
   for (var k in f) if (f[k]) v_inline_error_clear(k, val_hash, form);
   if (! e.length) return;
